!/*****************************************************************************/
! *
! *  Elmer/Ice, a glaciological add-on to Elmer
! *  http://elmerice.elmerfem.org
! *
! * 
! *  This program is free software; you can redistribute it and/or
! *  modify it under the terms of the GNU General Public License
! *  as published by the Free Software Foundation; either version 2
! *  of the License, or (at your option) any later version.
! * 
! *  This program is distributed in the hope that it will be useful,
! *  but WITHOUT ANY WARRANTY; without even the implied warranty of
! *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! *  GNU General Public License for more details.
! *
! *  You should have received a copy of the GNU General Public License
! *  along with this program (in file fem/GPL-2); if not, write to the 
! *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
! *  Boston, MA 02110-1301, USA.
! *
! *****************************************************************************/
! ******************************************************************************
FUNCTION Buelerprofile ( Model, nodenumber, coord) RESULT(elevation)
   USE Types
   USE DefUtils
   IMPLICIT NONE
   TYPE(Model_t), INTENT(IN) :: Model
   INTEGER, INTENT(IN) :: nodenumber
   REAL(KIND=dp), INTENT(IN) :: coord(3)
   REAL(KIND=dp) ::elevation
   ! ---------------------
   REAL(KIND=dp), PARAMETER :: n = 3.0_dp
   REAL(KIND=dp) :: L,H0,minh
   REAL(KIND=dp) :: T1, T2, H, R, x0, y0
   INTEGER :: DIM
   TYPE(Element_t), POINTER ::  BoundaryElement
   TYPE(ValueList_t), POINTER :: BC
   LOGICAL :: Found
   !----------------------
   
   DIM = CoordinateSystemDimension()

   ! get element and boundary condition
   BoundaryElement => Model % CurrentElement
   IF (.NOT.ASSOCIATED(BoundaryElement))THEN
     CALL Fatal('Buelerprofile', 'Boundary Element not found')
   END IF
   BC => GetBC(BoundaryElement)
   IF (.NOT.ASSOCIATED(BC))THEN
     CALL Fatal('Buelerprofile', 'No BC found')
   END IF
   
   ! get parameters
   L = GetConstReal(BC,"Bueler Length", Found)
   IF (.NOT.Found) &
        CALL FATAL('Buelerprofile','"Bueler Length" not found in boundary condition')
   
   H0 = GetConstReal(BC,"Bueler Height", Found)
   IF (.NOT.Found) &
        CALL FATAL('Buelerprofile','"Bueler Height" not found in boundary condition')

   minh = GetConstReal(BC,"Bueler Min Height", Found)
   IF (.NOT.Found) &
        CALL FATAL('Buelerprofile','"Bueler Min Height" not found in boundary condition')

   x0 = GetConstReal(BC,"Bueler X origin", Found)
   IF (.NOT.Found) x0=0.0_dp

   ! compute the radius with respect to the dimension of the problem
   SELECT CASE(DIM)
   CASE (3)
     y0 = GetConstReal(BC,"Bueler Y origin", Found)
     IF (.NOT.Found) y0=0.0_dp
     R = SQRT((coord(1) - x0)**2.0_dp  + (coord(2) - y0)**2.0_dp)
   CASE (2)
     R = SQRT((coord(1) - x0)**2.0_dp)
   CASE DEFAULT
     CALL FATAL('Buelerprofile','Wrong dimension of problem')
   END SELECT

   ! compute the elevation
   T1 = ( H0/((n - 1.0_dp)**(n/(2.0_dp*n + 2.0_dp))) )
   T2 = (n + 1.0_dp)*(R/L) - n*((R/L)**((n + 1.0_dp)/n)) + &
        n*((1.0_dp - (R/L))**((n + 1.0_dp)/n) ) - 1.0_dp
   H =  T1 * ( T2**(n/(2.0_dp*n + 2.0_dp)) )
   elevation = MAX(H,minh)
   
 END FUNCTION Buelerprofile
